
class OptionLoader {

    /**
     * @param {string} placeholder
     * @returns {string}
     */
    static generatePlaceholderOption(placeholder) {
        if (placeholder)
            return `<option value="0">${placeholder}</option>`;
        return '';
    }

    /**
     * @callback optionFactoryCallback
     * @param {object}
     */
    /**
     * @param {Object[]} data
     * @param {optionFactoryCallback} optionFactory
     */
    static constructSelectData(data, optionFactory) {
        let html = '';
        data.forEach(rec => html += optionFactory(rec));
        return html;
    };

    static stubFoo () {};

    constructor({$select, factory, url, genParams}) {
        this.$select = $select;
        this.factory = factory;

        this.url = URLdir + url;
        this.genParams = genParams;

        $select.change(() => this.onChange());

        // stubs
        this.onChange = OptionLoader.stubFoo;
        this.onReload = OptionLoader.stubFoo;
        this.placeholder = '';
    }

    reload() {
        $.postJSON(this.url, this.genParams(), data => {
            let html = this.placeholder + OptionLoader.constructSelectData(data, this.factory);
            this.$select.empty();
            this.$select.append($(html));
            this.onReload();
        });
    }

    setPlaceholder(placeholder) {
        this.placeholder = OptionLoader.generatePlaceholderOption(placeholder);
        return this;
    }

    setOnChange(callback) {
        this.onChange = callback;
        return this;
    }
    setOnReload(callback) {
        this.onReload = callback;
        return this;
    }
}


// ============================
//      groups
// ============================

/**
 * @var {optionFactoryCallback}
 * @param {number} data.ID
 * @param {number} data.GroupNum
 * @param {string} data.SpecName
 */
let getGroupOption = (data) => {
    let str = `группа ${data.GroupNum}`;
    if (data.SpecName) {
        str += " - ";
        str += data.SpecName;
    }
    return `<option value='${data.ID}'>${str}</option>`;
};

let getGroupLoader = ($select, genParams) => {
    let groupSelect = new OptionLoader({
        $select: $select,
        factory: getGroupOption,
        url: 'handler/map/GetGroups',
        genParams: genParams
    });
    groupSelect
        .setPlaceholder($select.children().html());
    return groupSelect;
};


// ============================
//      groups
// ============================

/**
 * @var {optionFactoryCallback}
 * @param {number} data.ID
 * @param {number} data.Num
 * @param {string} data.Degree
 */
let getGradeOption = (data) => {
    let title = (data.Degree == 'master') ? 'Магистратура' : 'Курс';
    let str = `${title} ${data.Num}`;
    return `<option value='${data.ID}'>${str}</option>`;
};

let getGradeLoader = ($select, genParams) => {
    let gradeSelect = new OptionLoader({
        $select: $select,
        factory: getGradeOption,
        url: 'handler/map/GetGrades',
        genParams: genParams
    });
    gradeSelect
        .setPlaceholder($select.children().html());
    return gradeSelect;
};
